@@ -1,6 +1,6 @@
 /*
  * AFM library public interface.
- * Copyright (c) 1995-1998 Markku Rossi.
+ * Copyright (c) 1995-1999 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
@@ -690,7 +690,7 @@ afm_font_encoding (AFMFont font, AFMEncoding enc, unsigned int flags)
 
 
 void
-afm_message (AFMHandle handle, int level, char *message)
+afm_message (AFMHandle handle, unsigned int level, char *message)
 {
   if (handle->verbose < level)
     return;
