@@ -1,6 +1,6 @@
 /*
  * Internal header for the AFM library.
- * Copyright (c) 1995-1998 Markku Rossi.
+ * Copyright (c) 1995-1999 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
@@ -270,7 +270,7 @@ extern AFMEncodingTable afm_koi8_encoding[];
  */
 
 /* Print message if <level> is larger than library's verbose level. */
-void afm_message ___P ((AFMHandle handle, int level, char *message));
+void afm_message ___P ((AFMHandle handle, unsigned int level, char *message));
 
 /* Print error message to stderr. */
 void afm_error ___P ((AFMHandle handle, char *message));
