@@ -369,10 +369,10 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
 	case kStartCharMetrics:
 	  GET_VALUE (AFM_TYPE_INTEGER);
 	  font->num_character_metrics = node.u.integer;
-	  font->character_metrics =
-	    (AFMIndividualCharacterMetrics *)
-	      calloc (font->num_character_metrics,
-		      sizeof (AFMIndividualCharacterMetrics));
+	  font->character_metrics
+	    = ((AFMIndividualCharacterMetrics *)
+	       calloc (font->num_character_metrics + 1,
+		       sizeof (AFMIndividualCharacterMetrics)));
 	  if (font->character_metrics == NULL)
 	    parse_error (handle, AFM_ERROR_MEMORY);
 
@@ -389,7 +389,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
 	      GET_VALUE (AFM_TYPE_INTEGER);
 	      font->num_kern_pairs = node.u.integer;
 	      font->kern_pairs =
-		(AFMPairWiseKerning *) calloc (font->num_kern_pairs,
+		(AFMPairWiseKerning *) calloc (font->num_kern_pairs + 1,
 					       sizeof (AFMPairWiseKerning));
 	      if (font->kern_pairs == NULL)
 		parse_error (handle, AFM_ERROR_MEMORY);
@@ -413,7 +413,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
 	      GET_VALUE (AFM_TYPE_INTEGER);
 	      font->num_track_kerns = node.u.integer;
 	      font->track_kerns
-		= (AFMTrackKern *) calloc (font->num_track_kerns,
+		= (AFMTrackKern *) calloc (font->num_track_kerns + 1,
 					   sizeof (AFMTrackKern));
 	      if (font->track_kerns == NULL)
 		parse_error (handle, AFM_ERROR_MEMORY);
@@ -441,7 +441,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
 	      GET_VALUE (AFM_TYPE_INTEGER);
 	      font->num_composites = node.u.integer;
 	      font->composites
-		= (AFMComposite *) calloc (font->num_composites,
+		= (AFMComposite *) calloc (font->num_composites + 1,
 					   sizeof (AFMComposite));
 	      if (font->composites == NULL)
 		parse_error (handle, AFM_ERROR_MEMORY);
@@ -988,7 +988,7 @@ read_composites (AFMHandle handle, ParseCtx *ctx, AFMFont font)
       GET_VALUE (AFM_TYPE_INTEGER);
       cm->num_components = node.u.integer;
       cm->components
-	= (AFMCompositeComponent *) calloc (cm->num_components,
+	= (AFMCompositeComponent *) calloc (cm->num_components + 1,
 					    sizeof (AFMCompositeComponent));
 
       /* Read composite components. */
