@@ -1,6 +1,6 @@
 /*
  * The main for states.
- * Copyright (c) 1997-1998 Markku Rossi.
+ * Copyright (c) 1997-2000 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
@@ -225,7 +225,7 @@ main (argc, argv)
 
   init_primitives ();
 
-  re_set_syntax (RE_SYNTAX_GNU_AWK);
+  re_set_syntax (RE_SYNTAX_GNU_AWK | RE_INTERVALS);
 
   /* Enter some system variables. */
   enter_system_variable ("program", program);
